SystemOrganization addCategory: #MuODeployement!


Object subclass: #MuODeployer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MuODeployement'!

MuODeployer class
	instanceVariableNames: ''!


!MuODeployer commentStamp: 'spfa 5/6/2006 23:31' prior: 0!
Responsible for the deployement of µO

- generation of a *.sar archive from a development image
- installation from *.sar into a fresh image
- saving of a public image from a development image
- transformation of a public image into a TCP/IP-driven widget
!

!MuODeployer class methodsFor: 'welcoming' stamp: 'spfa 4/24/2008 18:46'!
aboutToEnterMuOWorld

	(World hasProperty: #wasEntered)
		ifTrue: [^ self].

	self firstTimeActions ifTrue: [World getProperty: #wasEntered]! !

!MuODeployer class methodsFor: 'muO SAR release' stamp: 'spfa 1/26/2007 11:01'!
additionalFiles

	"MuODeployer additionalFiles"

	| flist |

	flist _ Set new.

	Preferences includeBooksInMuORelease 
		ifTrue: [flist addAll: (MuO booksForRelease 
									select: [:d | d fileExists]
									thenCollect: [:d | d file])].

	^ flist! !

!MuODeployer class methodsFor: 'ancilliary packages' stamp: 'spfa 5/22/2006 08:46'!
fileOutFunctionalTalk

	ChangeSetPackageExporter fileOutPackageNamed: 'FunctionalTalk'.! !

!MuODeployer class methodsFor: 'ancilliary packages' stamp: 'spfa 5/22/2006 15:17'!
fileOutMuOMaths

	SARPackageDumper fileOutPackage: MuOMathsInfo new 
		as: ('muOmaths.%s.sar' sformat: MuOMathsInfo version) ! !

!MuODeployer class methodsFor: 'welcoming' stamp: 'spfa 4/24/2008 23:10'!
firstTimeActions

	| menu book |

	"clean up"
	(FileDirectory default fileExists: 'muO welcome.pr')
		ifTrue: ['muO welcome.pr' asFileReference kill].

	menu _ MuO mainMenu stayUp: true. 
	menu openInWorld; bottomRight: World center.

	book _  (FileDirectory default 
				fullNameFor: 'muo books\DocBook entry points.morph').
	(FileDirectory default fileExists: book)
		ifTrue: [Morph fromLocalFileName: book].

	self inform: 'Welcome to muO !!
-
This is the first time you enter this project.
For a start please read the ''Entry points to muO'' book, then play with 
the ''muO objects'' menu (see notably the ''live documentation'' submenu).
-'.

	^ true! !

!MuODeployer class methodsFor: 'snippets' stamp: 'spfa 1/12/2006 19:26'!
incrementMuOVersion

	(self confirm: ('µO version jump to %s ?' sformat: MuOInfo version + 1))
		ifFalse: [^ self].

	MuOInfo class compile: ((
'
version

	^ %s
'
					) sformat: (MuOInfo version + 1))! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/11/2007 16:32'!
installAll: aListOfUrls

	^	aListOfUrls do: [:url |
			| smp dir fname |
			dir _ self packagesCacheDirectory.
			dir assureExistence.
			fname _ url path last.
			(dir fileExists: fname)
				ifFalse:
				[	| stream file |
					stream _ url retrieveContents contentStream binary.
					file _ dir newFileNamed: fname.
					file nextPutAll: stream contents.
					file close
				].
			fname _ dir fullNameFor: fname.
			[
				(fname findTokens: '.') last asUppercase caseOf:
				{	
					['MCZ'] -> [MczInstaller installFileNamed: fname].
					['SAR'] -> [SARInstaller installSAR: fname]
				}
			] on: Error
			do: 
			[
				(self confirm: 'Installation of ', fname, ' failed. Proceed anyway ?')
					ifFalse: [self error]
			]
		].! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/21/2007 17:04'!
installRequiredPackages

	| smf |
"
	smf _ PopUpMenu 
		confirm: 'muO: install required packages via SqueakMap ?'
		trueChoice: 'yes, use SqueakMap (locally in /sm)'
		falseChoice: 'no, directly fetch each package (locally in /muo-cache)'.
"
	smf _ false. "SqueakMap support broken at the moment..."

	smf 
		ifTrue: [self smInstallAll: self requiredPackagesIds]
		ifFalse: [self installAll: self requiredPackagesUrls]! !

!MuODeployer class methodsFor: 'muO image release' stamp: 'spfa 8/2/2008 18:23'!
makeImageRelease

	self releaseImageAs: 'muO.image'
			enable: {
						#enableMuOManagers .
						#stickToManager .
						#surmulotComponent . 

						#editFilesWithEmacs .
						#editTextsWithEmacs .
						#emacsSocketIsADaemon .
						#listenToEmacsAtStartup .
						#silentlyIgnoreErrorsFromEmacs .

						#enableCsoundInteraction .
	"					#enableLowkeyInteraction ."
						#enableSoundEditorInteraction .
						#enableSoundPlayerInteraction .
						#enableMF2TInteraction .
						#enableT2MFInteraction .
						#enableTimidityInteraction .

						#duplicateControlAndAltKeys .
					}
			disable: {
						#enableLowkeyInteraction .
						#enableMIDIEditorInteraction .

						#logEmacsSession.

						#warnIfNoSourcesFile .
						#warnIfNoChangesFile .

						#preserveTrash .
						#slideDismissalsToTrash . 						#syntaxHighlightingAsYouTypeAnsiAssignment .
					}! !

!MuODeployer class methodsFor: 'widget image setup' stamp: 'spfa 3/18/2008 21:10'!
makeWidget: aMorph extent: aPoint port: aNumber

	"Transform the current image into an emacs widget listening to a local port"

	SmalltalkImage current closeSourceFiles.
	SmalltalkImage current primImageName: ('widget %s' sformat: aNumber) .

	EmacsInteraction startOnPort: aNumber.
	ChangeSet newChanges:
		(ChangeSorter basicNewChangeSet: 'widget changeset'). 

	DisplayScreen depth: 32 width: aPoint x height: aPoint y fullscreen: false.
	Flaps disableGlobalFlaps: false.

	World submorphs do: [:m | m delete].

	World layoutPolicy: ProportionalLayout new.

	aMorph behaveAsSurmulotWidget

! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/12/2006 18:50'!
packagesCacheDirectory

	^	FileDirectory default directoryNamed:  'muo-cache'! !

!MuODeployer class methodsFor: 'muO image release' stamp: 'spfa 1/20/2007 13:58'!
releaseImageAs: releaseName enable: truePrefs disable: falsePrefs

"... voir ReleaseBuilder>>makeSqueaklandRelease pour inspiration"

	SmalltalkImage current saveAs: releaseName. 
	MuO cleanUp.
	MIDIOutputRecorder reset.

	"StarBrowser clean-up:"
	[ExtentionalClassification root removeAll] on: Error do: [].

	"Launcher initialization"
	ProjectLauncher deactivate.
	MuOLauncher activate.

	"Morphic references hide there:"
	WorldState deferredUIMessages flush.
	SurmulotManager killEmacsFlap; killKeykitFlap.
	HandMorph allInstances do: [:h | h pasteBuffer: nil].
	BlobMorph deleteAll.
	SketchEditorMorph allInstances do: [:m | m clearEnclosingPasteUpMorph].

	PluggableTextMorph allSubInstancesDo: 
		[:ptm | ptm hasUnacceptedEdits: false].

	ZoomableFieldMorph allSubInstancesDo: [:m | m releaseCachedState].
	MGraphElementMorph allSubInstancesDo: [:m | m releaseCachedState].
	MGraphBook allSubInstancesDo: [:m | m releaseCachedState].
	SketchMorph allSubInstancesDo: [:m | m releaseCachedState].
	ThumbnailMorph allSubInstancesDo: [:m | m releaseCachedState].

	Player freeUnreferencedSubclasses.
	Smalltalk cleanOutUndeclared.
	Behavior flushObsoleteSubclasses.
	Smalltalk garbageCollect; garbageCollect.
	DataStream initialize.
	CommandHistory resetAllHistory.

	Smalltalk garbageCollect.
	
	truePrefs	do: [:pref | Preferences enable:  pref].
	falsePrefs	do: [:pref | Preferences disable:  pref].

	SurmulotManager defineExternalEnvironment.
	Utilities initializeCommonRequestStrings.

	Smalltalk garbageCollect.
	Smalltalk forgetDoIts.
	Smalltalk cleanOutUndeclared.
	Smalltalk garbageCollect.

	Smalltalk flushClassNameCache.
	3 timesRepeat: 
			[Smalltalk garbageCollect.
			Symbol compactSymbolTable].

	MuO cleanUp.
"
	(self confirm: 'condense changes files ?')
		ifTrue: [Smalltalk condenseChanges].
"
	
	Csound options: '-+rtaudio=mme -d'.    "for csound 5 on windows !!"

	Utilities authorInitialsPerSe ifEmpty:
		[Utilities setAuthorInitials: 'muOuser'].

	SmalltalkImage current snapshot: true andQuit: true! !

!MuODeployer class methodsFor: 'muO SAR release' stamp: 'spfa 3/17/2007 23:08'!
releaseMuOPackage

	(self confirm: ('release muO.%s.sar ?' sformat: MuOInfo version))
		ifFalse: [^ self].

	MuO cleanUp.

	Preferences includeBooksInMuORelease 
		ifTrue: [self saveAllBooks].

	MuO compileReleaseBooks.

	ChangeSetPackageExporter fileOutPackageNamed: 'MuODeployement'.
"	ChangeSetPackageExporter fileOutPackageNamed: 'MuO39-hacks'." 

	SARPackageDumper fileOutPackage: MuOInfo new 
		as: ('muO.%s.sar' sformat: MuOInfo version) ! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/12/2006 18:50'!
requiredPackages

	"required before installing µO in a 3.8 image"

	^ #(
			'Connectors'
			'OSProcess'	
			'Regular Expression Plugin'
			'PreferenceBrowser'
			'OSC'
			'Network-HTML'
			'Scamper'
			'ExternalWebBrowser'
			'InstanceEncoder'
			'Shout'
		)! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/16/2006 21:08'!
requiredPackagesIds

	^	SystemVersion current majorMinorVersion caseOf: 
	{
		[ 'Squeak3.8'] ->
		[#(
			'88a466ca-e149-407f-9c0a-514f87dfeac1'
			'64b40e65-f1e4-4391-b502-3cd995f10cd4'
			'aec020d8-5489-4fbd-8087-3e5f1a2feb0e'
			'd4d23b62-ad79-42b1-8a70-4b4a0dba3ee1'
			'7cca283f-c529-4872-af58-fee7006d2068'
			'4eb612c8-d3a3-4d6b-8002-e0e5797d7276'
			'16ef4c23-5da6-4eea-a0e7-8518b3c93766'
			'1cb25c7f-feb5-4ba7-be91-94ac9b4cf86b'
			'0a05389f-ec46-411d-a162-6245461c083b'
			'3f468380-cd8a-4850-92e2-04dd2033fbc7'
			)].

		[ 'Squeak3.9'] ->
		[#(
			'88a466ca-e149-407f-9c0a-514f87dfeac1' 
			'64b40e65-f1e4-4391-b502-3cd995f10cd4'
			'aec020d8-5489-4fbd-8087-3e5f1a2feb0e'
			'0b4510fd-9793-4978-985b-1e80d5df906c'
			'7cca283f-c529-4872-af58-fee7006d2068'
			'4eb612c8-d3a3-4d6b-8002-e0e5797d7276'
	"		'16ef4c23-5da6-4eea-a0e7-8518b3c93766'   " "no scamper"
			'1cb25c7f-feb5-4ba7-be91-94ac9b4cf86b'
			'0a05389f-ec46-411d-a162-6245461c083b'
			'3f468380-cd8a-4850-92e2-04dd2033fbc7'
			)].
	}! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 3/29/2007 22:23'!
requiredPackagesUrls

	"3.8 and 3.8.1:"

	(SystemVersion current majorMinorVersion = 'Squeak3.8') ifTrue: 
		[^ #(
'http://map.squeak.org/accountbyid/092daeeb-7a37-427f-8afb-66bf2db74f98/files/ConnectorsFor38-185.sar'

'http://minnow.cc.gatech.edu/squeak/uploads/708/OSProcessV4-0-1.1.sar'

'http://map1.squeakfoundation.org/sm/accountbyid/092daeeb-7a37-427f-8afb-66bf2db74f98/files/Regex5.sar'

'http://www.squeaksource.com/PreferenceBrowser/PreferenceBrowser-hpt.22.mcz'

'http://www.squeaksource.com/OSCClient/OSC-mga.10.mcz'

'http://map1.squeakfoundation.org/sm/accountbyid/a6930213-b578-49b1-862e-228cc5ab39e7/files/Network-HTML-md.4.mcz'

'http://map1.squeakfoundation.org/sm/accountbyid/a6930213-b578-49b1-862e-228cc5ab39e7/files/Scamper-md.3.mcz'

'http://kilana.unibe.ch:8888/ExternalWebBrowser/ExternalWebBrowser-dd.7.1.mcz'

'http://www.squeaksource.com/TestBrowser/TestBrowser-tbn.119.mcz'

'http://squeak.saltypickle.com/InstanceEncoder/InstanceEncoder-jrp.9.mcz'

'http://map1.squeakfoundation.org/sm/accountbyid/46dcf6af-067d-43e3-9fc9-d7010e067153/files/shout4.sar') 
		collect: [:surl | 
						HttpUrl absoluteFromText: surl]
	].

	"3.9 and newer:"

		^ #(

'http://map.squeak.org/accountbyid/092daeeb-7a37-427f-8afb-66bf2db74f98/files/ConnectorsFor38-185.sar'

'http://minnow.cc.gatech.edu/squeak/uploads/708/OSProcessV4-0-1.1.sar'

'http://map1.squeakfoundation.org/sm/accountbyid/092daeeb-7a37-427f-8afb-66bf2db74f98/files/Regex5.sar'
"
'http://kilana.unibe.ch:8888/PreferenceBrowser/PreferenceBrowser-hpt.27.mcz'
"
'http://www.squeaksource.com/OSCClient/OSC-mga.10.mcz'

'http://map.squeak.org/accountbyid/a6930213-b578-49b1-862e-228cc5ab39e7/files/HTML-md.1.mcz'

'http://www.squeaksource.com/Scamper/Scamper-kfr.12.mcz'
"
'http://kilana.unibe.ch:8888/ExternalWebBrowser/ExternalWebBrowser-dd.7.1.mcz'
"
'http://www.squeaksource.com/TestBrowser/TestBrowser-tbn.119.mcz'

'http://squeak.saltypickle.com/InstanceEncoder/InstanceEncoder-jrp.9.mcz'

'http://map1.squeakfoundation.org/sm/accountbyid/46dcf6af-067d-43e3-9fc9-d7010e067153/files/shout4.sar') 
	collect: [:surl | 
		HttpUrl absoluteFromText: surl]! !

!MuODeployer class methodsFor: 'snippets' stamp: 'spfa 1/13/2006 21:58'!
resetStrikeFontsEmphasis

	"fix a weird problem in my development image - spfa"

	(StrikeFont allInstances select: [:f | 
		f name last isDigit and: [f emphasis isZero not]] )
	do: [:f | f emphasis: 0]! !

!MuODeployer class methodsFor: 'snippets' stamp: 'spfa 4/11/2008 21:41'!
saveAllBooks

	self resetStrikeFontsEmphasis.

	Utilities informUserDuring:[:blah |
		(MGraphBook allInstances 
			select: [:book | book isDocumentation 
								& book hasUsualFile])
			do: [:book |
					| n p |
					n _ book pageNumber.
					p _ book position.
					book center:  400@300.
					book goToPage: 1.
					blah value: book externalName.
					book saveOnUsualFile.
					book goToPage: n.
					book position: p]]! !

!MuODeployer class methodsFor: 'fresh image installation' stamp: 'spfa 1/12/2006 19:22'!
smInstallAll: aListOfIds

	[SMSqueakMap default reload] on: Error
		do: [SMSqueakMap default loadFull].

	^	aListOfIds do: [:id | SMSqueakMap default installPackageReleaseWithId: id]
! !
