SystemOrganization addCategory: #'People-spfa-files'!
SystemOrganization addCategory: #'People-spfa-files-Tests'!


FileList2 subclass: #FileBrowser
	instanceVariableNames: 'suffixes'
	classVariableNames: 'LastDirectories RecentDirectories'
	poolDictionaries: ''
	category: 'People-spfa-files'!

Object subclass: #FileReference
	instanceVariableNames: 'string lastAccessTime contents'
	classVariableNames: 'Files TmpFileSemaphore'
	poolDictionaries: ''
	category: 'People-spfa-files'!

TestCase subclass: #FileReferenceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'People-spfa-files-Tests'!

FileBrowser class
	instanceVariableNames: ''!

FileReference class
	instanceVariableNames: ''!

FileReferenceTest class
	instanceVariableNames: ''!


!FileBrowser commentStamp: 'spfa 5/16/2003 15:45' prior: 0!
Improved FileList2 : this one remembers the directories from where files were picked. those directories are stored in different lists, according to the files extensions

	"SPFAFileList modalFileSelectorForSuffixes: #('mid')"!

!FileReference commentStamp: 'spfa 11/13/2003 15:19' prior: 0!
Interface for referencing a file.

The idea is to work with files (considered mainly as data external to the image and as external programs binary locations) in a simple way.

some properties:

	- FileReferences are unique for any file name: they can not be copied. 

	- a FileReference store a lastAccessTime inst var allowing to detect if the file was modified on the disk by an external process.

	- a FileReference can also record the file content, enabling an image or project to swallow all externally referenced data so that it can be ported to another system.

	- there is support for temporary files (automatic naming and removal, periodic cleaning of garbage on the disk)
	
!

!FileReferenceTest commentStamp: '<historical>' prior: 0!
This is the unit test for the class FileReference. Unit tests are a good way to exercise the functionality of your system in a repeatable and automatic manner. They are therefore recommended if you plan to release anything. For more information, see: 
	- http://www.c2.com/cgi/wiki?UnitTest
	- http://minnow.cc.gatech.edu/squeak/1547
	- the sunit class category
!

!FileBrowser methodsFor: 'services' stamp: 'spfa 1/30/2003 22:53'!
addPath: aString
	"Add the given string to the list of recently visited directories."

	aString ifNil: [^self].

	"If a sub-directory is in the list, do nothing."
	(RecentDirectories detect: [ :aDir | ((aString, '*') match: aDir)] ifNone: [nil])
		ifNotNil: [^self].

	[RecentDirectories size >= 20]
		whileTrue: [RecentDirectories removeFirst].
	RecentDirectories addLast: aString! !

!FileReference methodsFor: 'testing' stamp: 'spfa 1/27/2004 13:22'!
alterEgo

	^ FileReference allInstances 
		detect: [:f | f entry = self entry]
		ifNone: [^ self]
! !

!String methodsFor: '*People-spfa-files' stamp: 'spfa 4/12/2007 09:36'!
asFileNameWithDulySeparator

	| fname |

	fname _ ((self copyReplaceAll: '/' with: FileDirectory slash)
		copyReplaceAll: '\' with: FileDirectory slash).
		"copyReplaceAll: ':' with: FileDirectory slash" 

"the last line was supposed to handle mac separators, but it clashes with Windows c:\ like path names"

	(FileDirectory slash == $:) ifTrue: "mac OS"
	[
		fname first == $: ifTrue: [fname _ fname allButFirst].
		self first isAlphaNumeric ifTrue: [fname _ ':', fname].
	].

	^ fname
! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/10/2003 17:49'!
asFileReference

	^ self! !

!String methodsFor: '*People-spfa-files' stamp: 'yo 11/2/2004 20:45'!
asFileReference

	^ FileReference for: self! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/13/2003 14:10'!
asString

	^ self fileName! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/28/2005 14:01'!
asUrl

	^ FileUrl pathParts: (self directory pathParts copyWith: self localName)! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 14:28'!
baseName
		
	^ FileDirectory baseNameFor: self fileName! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/13/2003 22:40'!
binaryOverwriteWith: aString

	(CrLfFileStream fileNamed: self asString) binary nextPutAll: aString; close.

! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 9/2/2005 14:48'!
cleanUpFiles

	Preferences alwaysRemoveTemporaryFiles ifTrue:
		[self files keysAndValuesDo: [:k :v | 
			(v isNil and: [self isTmp: k])
				ifTrue: [
					MuOLog log: ('Deleting temporary file %Q' sformat: k).
					(FileDirectory forFileName: k) deleteFileNamed: k
				]]].

	self files keysAndValuesRemove: [:k :v | v isNil]! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/13/2003 13:46'!
clone
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 10/31/2004 14:12'!
contentsOf: aFile

	^ aFile asFileReference fileContents! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/13/2003 13:46'!
copy
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference methodsFor: 'copying' stamp: 'spfa 9/8/2005 11:00'!
copyAs: aFileReference

	| file1 file2 |
	file1 _ (self directory readOnlyFileNamed: self asString) binary.
	file2 _ (self directory fileNamed: aFileReference asString) binary.
	self directory copyFile: file1 toFile: file2.
	file1 close.
	file2 close.

	^ aFileReference! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 1/27/2004 20:07'!
create

	self exists ifFalse:
		[(CrLfFileStream fileNamed: self asString) close].
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 9/2/2005 15:12'!
createFileRootname: aString ext: extension inDirectory: dir

	self tmpSema critical:
	[
		| n file files | 

		files _ dir fileNames.

		n _ 100000000000 atRandom.
		[
			file _ aString, n asString, (FileDirectory dot), extension.
			files includes: file
		]
		whileTrue: [n _ 100000000000 atRandom].
					
		^ (FileReference for: dir pathName, (FileDirectory slash), file) create. 
		"otherwise that name may be attributed again !!"
	]


! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 22:55'!
csdFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 22:56'!
csdString

	^ self fileContents! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/13/2003 13:46'!
deepCopy
	"Answer with the receiver, because FileReferences are unique."! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 10/31/2004 14:11'!
deleteFile: aFile

	aFile asFileReference kill! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 10:39'!
dirPath
		
	^ FileDirectory dirPathFor: self asString! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 11:01'!
directory
		
	^ FileDirectory forFileName: self asString! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 5/24/2004 15:05'!
displayContents

	self exists ifTrue:
		[self fileContents displayWithLabel: self asString]
! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 3/25/2005 17:11'!
edit

	self exists ifFalse: [^ self].

	(Preferences editFilesWithEmacs
		and: [EmacsInteraction isAvailable]
		and: [EmacsInteraction wantsToEditFile: self])
	ifTrue:
		[^ EmacsInteraction editFile: self].

	FileList openEditorOn: self fileStream editString: nil! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 10/31/2004 14:11'!
editFile: aFile

	aFile asFileReference edit! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/13/2003 16:11'!
ensureFileExists: aFileName

	FileReference ensureFileExists: aFileName 
		  error: (aFileName asString, ' does not exist')! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/6/2003 10:27'!
ensureFileExists: aFileName else: aBlock

	(self fileExists: aFileName)
		ifFalse: [^ aBlock value].
! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/6/2003 10:43'!
ensureFileExists: aFileName elseWrite: aString

	FileReference ensureFileExists: aFileName 
		  else: [self writeFile: aFileName withContents: aString]! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/6/2003 10:27'!
ensureFileExists: aFileName error: aString

	(self fileExists: aFileName)
		ifFalse: [self error: aString].
! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/6/2003 10:44'!
ensureFileExists: aFileName error: aString thenWait: millisecs

	FileReference ensureFileExists: aFileName error: aString.

	(Delay forMilliseconds: millisecs) wait.! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 11/6/2003 10:27'!
ensureFilesExist: aSetOfFileName

	aSetOfFileName do: [:ea | self ensureFileExists: ea].
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 13:38'!
entry 

	^ FileDirectory directoryEntryFor: self asString! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 11/13/2003 15:47'!
executableExtensions

	"we should be able to extend this list easily..."

	^ #('exe' 'bat' 'com')
		

! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/13/2003 14:09'!
exists

	^ self class fileExists: self fileName! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/13/2003 14:09'!
existsOrFails

	self class ensureFileExists: self fileName.! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/13/2003 14:09'!
existsOrFails: aString

	self class ensureFileExists: self fileName error: aString.! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 14:25'!
extension
		
	^ FileDirectory extensionFor: self fileName! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/13/2003 14:07'!
fileContents

	self exists ifFalse: [^ nil].

	^ (CrLfFileStream readOnlyFileNamed: self asString) contentsOfEntireFile.! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 6/8/2004 17:00'!
fileExists: aFileName 

	^ ((aFileName notNil) and: 
	[(FileDirectory default) fileExists: aFileName asString])
	
! !

!FileReference methodsFor: 'about' stamp: 'spfa 1/26/2007 11:05'!
fileName

	"always return a full name !!"

	^ FileDirectory default fullNameFor: 
		(string ifNil: [^ '']) asFileNameWithDulySeparator! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/13/2003 15:24'!
fileName: aString

	string _ aString.
	lastAccessTime _ nil.
	self forgetContents! !

!FileReference class methodsFor: 'testing existence' stamp: 'spfa 10/29/2004 14:52'!
fileSize: aFileName 

	^ ((FileDirectory forFileName: aFileName asString) 
		entryAt: (FileDirectory localNameFor: aFileName asString)) fileSize
	
! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 11/10/2003 18:06'!
fileStream

	self exists ifTrue:
		[^ FileDirectory default fileNamed: self asString].

	self error: 'this file does not exists'! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 6/20/2004 21:21'!
files

	^ Files ifNil: [Files _ WeakValueDictionary new]! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 1/13/2006 16:12'!
for: aFileName

	| actualName |
	
	actualName _ aFileName.

	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: [actualName _ actualName asLowercase].

	((self files includesKey: actualName)
		and: [(self files at: actualName) notNil])
		ifTrue: [^ self files at: actualName]
		ifFalse: [ | fref | 
					fref _ self new fileName: actualName.
					self files add: actualName -> fref.
					^ fref]
! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 11/13/2003 14:19'!
forgetContents

	contents _ nil
	

	! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/13/2003 15:44'!
fullExecutableName

	self possibleExecutableNames
		do: [:n | n asFileReference exists ifTrue: [^ n asFileReference fullName]].
		
	(self dirPath = '')
	ifTrue: 
		[^ (FileReference path collect: [:p |
			(self fullNameIn: (FileDirectory on: p)) asFileReference fullExecutableName])
			detect: [:n | n notNil] ifNone: [nil]].

	^ nil! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 15:21'!
fullName

	^ self fullNameIn: FileDirectory default
! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 15:23'!
fullNameIn: aFileDirectory

	^ aFileDirectory fullNameFor: self asString
! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 1/7/2005 18:45'!
getEnv: aString

	| var |

	OSProcess accessor canAccessSystem
		ifFalse: [self error: 'OSProcess unavailable'].

	var _ aString asUppercase.

	OSProcess thisOSProcess environment associationsDo:
		[:a | a key asUppercase = var
			ifTrue: [^ a value veryDeepCopy]].

	^ nil

! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 1/30/2003 22:48'!
initialize

	"SPFAFileList initialize"

	super initialize.

	LastDirectories _ Dictionary new.
	RecentDirectories _ OrderedCollection new.
	! !

!FileReference class methodsFor: 'initialization' stamp: 'spfa 1/20/2007 13:56'!
initialize

	self registerStringFormats
	! !

!FileReference methodsFor: 'testing' stamp: 'spfa 4/15/2005 18:49'!
isExecutable

	"note: this is different from OSProcess accessor isExecutable: aString"
	
	"this is quite slow. better avoid using this method at all..."

	(self dirPath = '')
	ifFalse:
	[
		self possibleExecutableNames
			do: [:n | ([FileDirectory default fileExists: n] on: Error do: [false]) 
						ifTrue: [^ true]].		
	]
	ifTrue: 
		[FileReference path do: 
			[:p |
				 (self fullNameIn: (FileDirectory on: p)) asFileReference isExecutable
			 	 ifTrue: [^ true]]].

	^ false
! !

!FileReference class methodsFor: 'testing' stamp: 'spfa 11/6/2003 16:34'!
isExecutable: aString

	"note: this is different from OSProcess accessor isExecutable: aString"

	aString ifNil: [^ false].

	{ aString . aString, '.bat' . aString, '.exe' . aString, '.com'}
	do: [:n | (self fileExists: n) ifTrue: [^ true]].
		
	(FileDirectory dirPathFor: aString) = ''
	ifTrue: 
		[self path do: 
			[:p |
				 (self isExecutable: ((FileDirectory on: p) fullNameFor: aString))
			 	 ifTrue: [^ true]]].

	^ false
! !

!FileReference methodsFor: 'testing' stamp: 'spfa 9/7/2005 18:22'!
isFile

	^ true! !

!Object methodsFor: '*People-spfa-files' stamp: 'spfa 9/7/2005 18:22'!
isFile

	^ false! !

!FileReference methodsFor: 'testing' stamp: 'spfa 2/2/2004 13:39'!
isFileReference

	^ true! !

!Object methodsFor: '*People-spfa-files' stamp: 'spfa 2/2/2004 13:40'!
isFileReference

	^ false! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/13/2003 14:19'!
isInlined

	^ contents notNil! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/10/2003 17:42'!
isReferencedElsewhere

	"comment implmenter a au plus juste ??"
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 9/2/2005 14:49'!
isTmp: aFile

	| tmpRoot |

	tmpRoot _ self tmpFileNameRoot.
	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: [tmpRoot _ tmpRoot asLowercase].
	^ (FileDirectory localNameFor: aFile asString) beginsWith: tmpRoot! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 9/8/2005 10:57'!
kill

	self exists
	ifTrue:
		[self directory deleteFileNamed: self asString].
! !

!FileReference methodsFor: 'about' stamp: 'spfa 10/26/2004 21:17'!
lastAccessTime

	^ 
lastAccessTime ! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 1/30/2003 18:15'!
lastDirectories

	^ LastDirectories 
	! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 14:27'!
localName
		
	^ FileDirectory localNameFor: self fileName! !

!FileReference methodsFor: 'about' stamp: 'spfa 12/20/2005 14:14'!
localNameSansExtension
		
	^ FileDirectory baseNameFor: (FileDirectory localNameFor: self fileName)! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 22:55'!
midiFile

	^ self ! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/7/2003 15:30'!
modalFileSelectorForSuffixes: aList inDir: aDirectory

	| window aFileList |

	window _ self morphicViewFileSelectorForSuffixes: aList inDir: aDirectory.
	aFileList _ window valueOfProperty: #fileListModel.
	window openCenteredInWorld.
	[window world notNil] whileTrue: [
		window outermostWorldMorph doOneCycleNow.
	].
	^aFileList getSelectedFile! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 13:41'!
modificationTime 

	^ self entry modificationTime! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/7/2003 15:29'!
morphicViewFileSelectorForSuffixes: aList

	^ self morphicViewFileSelectorForSuffixes: aList inDir: nil! !

!FileBrowser class methodsFor: 'as yet unclassified' stamp: 'spfa 11/7/2003 15:28'!
morphicViewFileSelectorForSuffixes: aList inDir: aDirectory
	"Answer a morphic file-selector tool for the given suffix list"

	| dir aFileList window fixedSize midLine gap |

	aDirectory ifNotNil: [dir _ aDirectory]
	ifNil:
	[

	aList 
	ifNil: [dir _ FileDirectory default]
	ifNotNil: 
		[dir _ LastDirectories at: aList 
									  ifAbsent: [FileDirectory default]].
	].
	aFileList _ self new directory: dir.
	aFileList suffixes: aList.

	aFileList optionalButtonSpecs: aFileList okayAndCancelServices.
	aList ifNotNil:
		[aFileList fileSelectionBlock: [:entry :myPattern |
			entry isDirectory
				ifTrue:
					[false]
				ifFalse:
					[aList includes: (FileDirectory extensionFor: entry name asLowercase)]] fixTemps].
	window _ BorderedMorph new
		layoutPolicy: ProportionalLayout new;
		color: Color tan muchLighter;
		borderColor: Color gray;
		borderWidth: 2;
		layoutInset: 4;
		extent: 600@400;
		useRoundedCorners.
	window setProperty: #fileListModel toValue: aFileList.
	aFileList modalView: window.
	midLine _ 0.4.
	fixedSize _ 25.
	gap _ 1.
	self addFullPanesTo: window from: {
		{self textRow: 'Please select a file'. 0 @ 0 corner: 1 @ 0. 0@0 corner: 0@fixedSize}.
		{aFileList optionalButtonRow. 0 @ 0 corner: 1 @ 0. 0@fixedSize corner: 0@(fixedSize * 2)}.
		{aFileList morphicDirectoryTreePane. 0@0 corner: midLine@1. 
					gap @(fixedSize * 2) corner: gap negated@0}.
		{aFileList morphicFileListPane. midLine @ 0 corner: 1@1. 
					gap@(fixedSize * 2) corner: gap negated@0}.
	}.


	aFileList postOpen.
	aFileList resort: #name.

	^ window ! !

!FileReference methodsFor: 'about' stamp: 'spfa 11/13/2003 14:20'!
noteAccessTime

	
lastAccessTime _ Date today asSeconds + Time now asSeconds! !

!FileReference methodsFor: 'private' stamp: 'spfa 6/17/2004 14:13'!
objectForDataStream: refStrm
    "Return an object to store on an external data stream."

	"  revoir compltement"
	
	^ self 
"
    self isInlined ifTrue: [^ self]
		ifFalse: [(self confirm: 'do we inline ', self fullName, ' ?')
				 ifTrue: [^ self swallowContents]
				 ifFalse: [^ self]]
"! !

!FileBrowser methodsFor: 'hits' stamp: 'spfa 1/30/2003 22:51'!
okHit
	
	self class lastDirectories add: (suffixes -> self directory).
	self addPath: directory pathName.
	super okHit! !

!FileBrowser methodsFor: 'services' stamp: 'spfa 1/30/2003 22:13'!
okayAndCancelServices
	"Answer ok and cancel services"

	^ {self serviceOkay. self serviceRecentDirs . self serviceCancel}! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 18:30'!
orchestraFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 18:15'!
orchestraString

	^ self fileContents! !

!FileReference methodsFor: 'accessing' stamp: 'spfa 6/23/2004 14:08'!
overwriteWith: aString

	(CrLfFileStream fileNamed: self asString) nextPutAll: aString; close.

! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 10/21/2004 08:18'!
path

	| path |

	path _ self getEnv: 'PATH'.
	path ifNil: [self error: 'no PATH detected !!'].

	^ ((path replaceAll: $ with: $&) replaceAll: $; with: $ ) substrings
		collect: [:d | d replaceAll: $& with: $ ]

		

! !

!FileDirectory class methodsFor: '*People-spfa-files' stamp: 'spfa 5/5/2006 22:59'!
pathParts: anOrderedCollection

	^ self on: (anOrderedCollection inject: '' into: [:path :part  | 
						path, part , self pathNameDelimiter asString])! !

!FileReference methodsFor: 'private' stamp: 'spfa 11/13/2003 15:36'!
possibleExecutableNames

	^ {self fileName}, (self class executableExtensions collect: [:x |
		FileDirectory fileName: self fileName extension: x])! !

!FileReference methodsFor: 'printing' stamp: 'spfa 1/26/2007 11:06'!
printOn: aStream

	string 
		ifNil: [aStream nextPutAll: '[-void-]']
		ifNotNil: [aStream nextPut: $[; nextPutAll: (string ifNil: ['']) asFileNameWithDulySeparator; nextPut: $]] 
! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 12/18/2005 21:21'!
query: aString

	"FileReference query: 'file name ?'"

	^ self query: aString directory: FileDirectory default
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 12/18/2005 21:21'!
query: aString directory: aDirectory

	^ self query: aString directory: aDirectory default: ''
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 12/20/2005 13:52'!
query: aString directory: aDirectory default: bString

	^ (((FileBrowser modalFolderSelector: aDirectory) ifNil:[^ nil]) pathName,
			FileDirectory slash,
	    		((FillInTheBlank request: aString initialAnswer: bString) 
				whenEmpty: [^ nil]))
		asFileReference
								! !

!FileReference class methodsFor: 'instance creation' stamp: 'spfa 12/18/2005 21:22'!
query: aString for: receiver getter: gSelector setter: sSelector

	| ddir dname got |

	got _ receiver perform: gSelector.
	ddir _ got ifNil: [FileDirectory default] ifNotNil: [got asFileReference directory].
	dname _ got ifNil: [''] ifNotNil: [got asFileReference localName].

	receiver perform: sSelector withArguments: 
		{(self query: aString directory: ddir default: dname) ifNil: [^ self]}
								! !

!FileReference methodsFor: 'private' stamp: 'spfa 1/27/2004 13:26'!
readDataFrom: aDataStream size: varsOnDisk
   
	super readDataFrom: aDataStream size: varsOnDisk.

	self isInlined ifTrue:
	[
		(self confirm: 'do we create ', self fullName, ' ?')
				 ifTrue: [self restoreContents]
	].

	self forgetContents.

	^ self alterEgo
! !

!FileBrowser methodsFor: 'hits' stamp: 'spfa 11/7/2003 15:13'!
recentDirsHit
	"Put up a menu and let the user select from the list of recently visited directories."

	| dirName |
	RecentDirectories isEmpty ifTrue: [^self].
	dirName := (SelectionMenu selections: RecentDirectories) startUp.
	dirName == nil ifTrue: [^self].
	self setSelectedDirectoryTo: (FileDirectory on: dirName).

! !

!FileReference class methodsFor: 'initialization' stamp: 'spfa 12/1/2005 13:42'!
registerStringFormats

	Smalltalk at: #StringFormat ifAbsent: [^ self].
	StringFormat replacementsDictionary  
		add: $F -> [:o :n | FileReference wrapFileName: o asString];
		add: $E -> [:o :n | FileReference wrapFileNameForEmacs: o asString].
	StringFormat helpDictionary  
		add: '%F' -> 'file reference';
		add: '%E' -> 'file reference for Emacs'.! !

!FileReference methodsFor: 'about' stamp: 'spfa 10/14/2005 11:51'!
relativeName

	"truly relative name, do not confuse with FileDirectory>>relativeNameFor: which does the opposite as what its name says"

	^ self relativeNameFor: FileDirectory default
! !

!FileReference methodsFor: 'about' stamp: 'spfa 1/20/2007 13:56'!
relativeNameFor: aDirectory

	"truly relative name, do not confuse with FileDirectory>>relativeNameFor: which does the opposite as what its name says"

	| pp dpp delim |

	pp _ self directory pathParts.
	dpp _ aDirectory pathParts.
	FileDirectory activeDirectoryClass isCaseSensitive
		ifFalse: 
		[
			pp _ pp collect: [:p | p  asLowercase]. 	
			dpp _ dpp collect: [:p | p  asLowercase]. 	
		].
	delim _ FileDirectory default pathNameDelimiter asString.

	(pp beginsWith: dpp)
		ifTrue: [^ ((pp allButFirst: dpp size) inject: '' into: [:name :p |
							name, p, delim]),
					self localName].

	^ '..', delim, (self relativeNameFor: aDirectory upperDirectory)
! !

!FileReference methodsFor: 'copying' stamp: 'spfa 12/16/2005 19:21'!
renamedAs: aFile

	aFile exists ifTrue: [aFile kill].
	Smalltalk garbageCollect.
	self directory rename: self asString toBe: aFile asString.

	^aFile! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 1/27/2004 14:04'!
restoreContents

	self isInlined ifFalse: [self error: 'no inlined contents'].
	
	((FileReference fileExists: self fullName) 
		and: [(self confirm: self fullName, ' already exists. Overwrite it ?') not])
	ifTrue: [^ self].

	(FileDirectory default directoryExists: self dirPath)
		ifFalse:
			[ "the path does not exists; change to FileDirectory default"
				string _ self localName.
			].

	self class withTmpFileExt: 'gz' do: [:zipfile |

		| zipped buffer unzipped |
	
		zipfile binaryOverwriteWith: contents.

		unzipped _ FileStream newFileNamed: self fullName.
		unzipped binary.
		zipped _ GZipReadStream on: zipfile fileStream.
		buffer _ ByteArray new: 50000.
		'Uncompressing ' , self fullName
			displayProgressAt: Sensor cursorPoint
			from: 0
			to: zipped sourceStream size
			during: 
				[:bar | 
					[zipped atEnd]
						whileFalse: 
						[bar value: zipped sourceStream position.
						unzipped nextPutAll: (zipped nextInto: buffer)].
					zipped close.
					unzipped close].

		self forgetContents]


	! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 22:55'!
scoreFile

	^ self ! !

!FileReference methodsFor: 'converting' stamp: 'spfa 9/7/2005 22:56'!
scoreString

	^ self fileContents! !

!FileReference methodsFor: 'testing' stamp: 'spfa 9/2/2005 14:49'!
seemsToBeATmpFile

	^ self class isTmp: self! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 12/20/2005 13:52'!
selectDirectory

	^ FileBrowser modalFolderSelector
! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 12/20/2005 13:52'!
selectFile

	| file |

	(file _ FileBrowser modalFileSelector) ifNil: [^nil].
	file close.

	^ file name asFileReference.
! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 11/7/2003 15:31'!
selectFile: aList

	^ self selectFile: aList inDir: nil	! !

!FileReference class methodsFor: 'selection' stamp: 'spfa 12/20/2005 13:52'!
selectFile: aList inDir: aDirectory

	| file |

	(file _ FileBrowser modalFileSelectorForSuffixes: aList inDir: aDirectory) ifNil: [^nil].
	file close.

	^ file name asFileReference.
! !

!FileBrowser methodsFor: 'services' stamp: 'spfa 1/30/2003 22:14'!
serviceRecentDirs 

	"Answer a service for quick selection of recently used directories"

	^ (SimpleServiceEntry new
		provider: self label: 'recentDirs' selector: #recentDirsHit 
		description: 'hit here to pop up the Recent Directories menu')
		extraSelector: nil;
		buttonLabel: 'recent dirs'! !

!FileReference methodsFor: 'testing' stamp: 'spfa 10/29/2004 14:54'!
size

	^ self class fileSize: self fileName! !

!FileBrowser methodsFor: 'suffixes' stamp: 'spfa 1/30/2003 18:13'!
suffixes: aList

	suffixes _ aList! !

!FileReference methodsFor: 'inlining' stamp: 'spfa 11/13/2003 22:44'!
swallowContents

	self noteAccessTime.

	self class withTmpFileExt: 'gz' do: [:zipfile |

		| zipped buffer stream |

		zipped _ GZipWriteStream on: zipfile fileStream binary.
		buffer _ ByteArray new: 50000.
		stream _ self fileStream.
		'Compressing ', self fullName displayProgressAt: Sensor cursorPoint
			from: 0 to: stream size
			during: [:bar |
				[stream atEnd] whileFalse: [
					bar value: stream position.
					zipped nextPutAll: (stream nextInto: buffer)].
				zipped close.
				stream close].

		contents _ zipfile fileContents]
	
	

	! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 1/20/2007 21:07'!
testFileExists

	self assert: (FileReference fileExists: SmalltalkImage current imageName).
	self deny: (FileReference fileExists: 'some dummy name.yop').
				  ! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/30/2005 12:08'!
testStringFormat

	self assert: ('%s -W -r %s -o %F' sformat: {'srconv' . 44100 . 'hi joe.wav'}) ='srconv -W -r 44100 -o "hi joe.wav"' 
				 ! !

!FileReferenceTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/13/2003 15:49'!
testTmpFile

	| tmpFile |

	self assert: ((FileReference withTmpFileDo: [:f | 
					tmpFile _ f.
					f overwriteWith: 'yo'.
					f fileContents]) = 'yo'
				 and: [tmpFile exists not] )
				 ! !

!FileReference class methodsFor: 'environment' stamp: 'spfa 1/26/2007 10:19'!
tmpDirectory

	| dir |

	dir _ ('tmp', FileDirectory slash) asFileReference directory. 
	dir exists ifFalse: [FileDirectory default createDirectory: 'tmp'].

	^ dir! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 1/20/2007 13:56'!
tmpFileExt: extension

	^ self tmpFileExt: extension inDirectory: self tmpDirectory.! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 9/2/2005 14:59'!
tmpFileExt: extension inDirectory: dir

	^ self createFileRootname: self tmpFileNameRoot ext: extension inDirectory: dir
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/13/2003 10:53'!
tmpFileExt: extension inSameDirectoryAs: aFileName

	^ self tmpFileExt: extension inDirectory: (FileDirectory forFileName: aFileName asString)

! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/7/2003 10:31'!
tmpFileNameExtension

	^ 'TMP'
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/7/2003 10:19'!
tmpFileNameRoot

	^ '_TMP_'! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 6/20/2004 21:50'!
tmpSema

	^ TmpFileSemaphore ifNil: 
		[TmpFileSemaphore _ Semaphore forMutualExclusion]! !

!FileDirectory methodsFor: '*People-spfa-files' stamp: 'spfa 5/5/2006 22:59'!
upperDirectory

	^ self class pathParts: self pathParts allButLast! !

!FileReference methodsFor: 'copying' stamp: 'spfa 11/13/2003 13:47'!
veryDeepCopyWith: deepCopier
	"Return self.  I can't be copied."! !

!FileReference methodsFor: 'testing' stamp: 'spfa 11/13/2003 14:12'!
wasModified

	^ (lastAccessTime isNil
			or: [self entry isNil]
			or: [self entry modificationTime > lastAccessTime])! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/7/2003 10:32'!
withTmpFileDo: aBlock

	^ self withTmpFileExt: self tmpFileNameExtension do: aBlock.
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 1/20/2007 13:56'!
withTmpFileExt: anExtension do: aBlock

	^ self withTmpFileExt: anExtension inDirectory: self tmpDirectory do: aBlock.
! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 11/13/2003 10:22'!
withTmpFileExt: anExtension inDirectory: dir do: aBlock

	| tmpFileRef returnValue |

	tmpFileRef _ FileReference tmpFileExt: anExtension inDirectory: dir.
	[returnValue _ aBlock value: tmpFileRef]

		"DEBUG:
					 tmpFileRef displayContents 
		"

		ensure: [tmpFileRef kill].

	^ returnValue! !

!FileReference class methodsFor: 'temporary files' stamp: 'spfa 12/2/2005 09:39'!
withTmpFileExt: anExtension inSameDirectoryAs: aFileReference do: aBlock

	^ self withTmpFileExt: anExtension inDirectory: aFileReference directory do: aBlock.

! !

!FileReference methodsFor: 'converting' stamp: 'spfa 11/14/2003 15:14'!
wrapFileName

	^ FileReference wrapFileName: self fileName! !

!FileReference class methodsFor: 'utils' stamp: 'spfa 11/13/2003 16:13'!
wrapFileName: aFileName

	((aFileName includes: $ )
		and: [(aFileName includes: $") not])
	ifTrue: [^ '"', aFileName, '"'].

	^ aFileName.! !

!FileReference class methodsFor: 'utils' stamp: 'spfa 3/14/2008 20:43'!
wrapFileNameForEmacs: aFileName

	^ '"', (aFileName copyReplaceAll: '\' with: '/'), '"'! !

!FileReference class methodsFor: 'actions' stamp: 'spfa 10/31/2004 14:11'!
writeFile: aFile withContents: aString 

	^ aFile asFileReference overwriteWith: aString

	! !

!String methodsFor: '*People-spfa-files' stamp: 'spfa 9/7/2005 18:27'!
writeToFile: aFileReference

	aFileReference overwriteWith: self! !
FileBrowser initialize!
FileReference initialize!
